﻿using System;
using System.Reflection;
using System.Reflection.Emit;

class Program
{
    static void Main(string[] args)
    {
        MyTemp instance = new MyTemp();

        SubscribeAtRuntime(instance);
        MyTempEventTest(instance);

        instance.Create();
    }

    private static void SubscribeAtRuntime(MyTemp instance)
    {
        Type targetType = instance.GetType();

        EventInfo ei = targetType.GetEvent("Created", BindingFlags.Public | BindingFlags.Instance);
        MethodInfo consoleWriteMethod = typeof(Console).GetMethod("Write", BindingFlags.Public | BindingFlags.Static, null, new Type[] { typeof(object) }, null);
        MethodInfo consoleWriteLineMethod = typeof(Console).GetMethod("WriteLine", BindingFlags.Public | BindingFlags.Static, null, new Type[] { typeof(object) }, null);
        FieldInfo nameField = typeof(TempEventArgs).GetField("Name", BindingFlags.Public | BindingFlags.Instance);

        DynamicMethod proxyMethod = new DynamicMethod(
            "Proxy_Created",
            typeof(void), new Type[] { typeof(object), typeof(TempEventArgs) });

        ILGenerator il = proxyMethod.GetILGenerator();

        // Console.Write("Event Fired: " );
        il.Emit(OpCodes.Ldstr, "Event Fired: ");
        il.Emit(OpCodes.Call, consoleWriteMethod);

        // Console.WriteLine(e.Name);
        il.Emit(OpCodes.Ldarg_1);
        il.Emit(OpCodes.Ldfld, nameField);
        il.Emit(OpCodes.Call, consoleWriteLineMethod);
        il.Emit(OpCodes.Ret);

        Type eventHandlerType = ei.EventHandlerType;
        eventHandlerType = typeof(EventHandler<TempEventArgs>);

        Delegate proxyDelegate = proxyMethod.CreateDelegate(eventHandlerType);
        ei.AddEventHandler(instance, proxyDelegate);
    }

    private static void Instance_Created(object sender, TempEventArgs e)
    {
        Console.WriteLine("Event Fired: " + e.Name);
    }

    private static void MyTempEventTest(object objTarget)
    {
        EventInfo ei = typeof(MyTemp).GetEvent("Created", BindingFlags.Public | BindingFlags.Instance);
        ei.AddEventHandler(objTarget, (EventHandler<TempEventArgs>)Instance_Created);
    }
}

public class TempEventArgs : EventArgs
{
    public string Name;
}

public class MyTemp
{
    public void Create()
    {
        Created(this, new TempEventArgs { Name = "MyTemp" });
    }

    public event EventHandler<TempEventArgs> Created;
}
